@@ -92,6 +92,10 @@ timeout = click.option(
     '--timeout', type=float, default=None, show_default=True,
     help='operation timeout (s)')
 
+stop_timeout = click.option(
+    '--stop-timeout', type=float, default=5, show_default=True,
+    help='timeout for the stop operation. If this timeout expires a kill is done')
+
 dserver = click.option(
     '--exclude-dserver/--include-dserver', default=True,
     show_default=True, is_flag=True, help='exclude/include DServer devices')
@@ -156,17 +160,26 @@ dry_run = click.option('--dry-run', default=False, show_default=True,
 
 class ClickOutputHandler(logging.Handler):
 
+    Style = collections.defaultdict(dict)
+    Style.update({
+        logging.DEBUG: dict(dim=True),
+        logging.INFO: dict(),
+        logging.WARNING: dict(fg='yellow'),
+        logging.ERROR: dict(fg='red'),
+        logging.CRITICAL: dict(fg='magenta')}
+    )
+
     def emit(self, record):
         try:
             record.name = record.name.replace('tangoctl.', '')
-            msg = self.format(record)
-            click.echo(msg)
+            record.msg = click.style(record.msg, **self.Style[record.levelno])
+            click.echo(self.format(record))
         except Exception:
             self.handleError(record)
 
 
 def _prepare_logging():
-    fmt = '%(asctime)s %(name)12s: %(message)s'
+    fmt = '%(asctime)s %(name)14s: %(message)s'
     handler = ClickOutputHandler()
     handler.addFilter(lambda record: record.name.startswith('tangoctl'))
     handler.setLevel(logging.INFO)
@@ -1153,8 +1166,10 @@ def starter_tree(starter, server, server_type, server_instance, all):
 @server_type_filter
 @server_instance_filter
 @timeout
+@stop_timeout
 @dry_run
-def starter_stop(starter, server, server_type, server_instance, timeout, dry_run):
+def starter_stop(starter, server, server_type, server_instance, timeout,
+                 stop_timeout, dry_run):
     """Stop selected servers.
 
     Servers which are reported to be stopped are skipped.
@@ -1171,7 +1186,8 @@ def starter_stop(starter, server, server_type, server_instance, timeout, dry_run
     try:
         tangoctl.starter_stop_servers(
             starter_name=starter, server_name=server, server_type=server_type,
-            server_instance=server_instance, timeout=timeout, dry_run=dry_run)
+            server_instance=server_instance, timeout=timeout,
+            stop_timeout=stop_timeout, dry_run=dry_run)
     except gevent.Timeout:
         click.secho('Operation timeout ({} s)'.format(timeout), fg='yellow')
 
