@@ -622,25 +622,38 @@ class Starter:
                 return state
             gevent.sleep(0.2)
 
-    def kill_server(self, name):
+    def kill_server(self, name, dry_run=False):
         state = self[name]['state']
         if state == 'FAULT':
             self._log.info('skipped killing %r (%s)',
                            name, human_starter_server_state(state))
             return True
         self._log.info('killing %r', name)
-        self.starter_device.HardKillServer(name)
-        state = self.wait_for_state(name, lambda s: s in ('MOVING', 'FAULT'))
-        if not state:
-            self._log.warn('failed to wait to kill %r', name)
-            return False
-        if state == 'FAULT':
-            self._log.info('killed %r', name)
-            return True
-        state = self.wait_for_state(name, lambda s: s == 'FAULT')
-        if not state:
-            self._log.warn('failed to kill %r', name)
-            return False
+        if not dry_run:
+            try:
+                self.starter_device.HardKillServer(name)
+            except tango.DevFailed as err:
+                err = err.args[0].desc
+                state = self[name]['state']
+                if state == 'FAULT':
+                    self._log.warning(
+                        'server %r was killed but reported: %s', name, err)
+                    return True
+                else:
+                    self._log.error('failed to kill %r: %s', name, err)
+                    return False
+            state = self.wait_for_state(
+                name, lambda s: s in ('MOVING', 'FAULT'))
+            if not state:
+                self._log.warn('failed to wait to kill %r', name)
+                return False
+            if state == 'FAULT':
+                self._log.info('killed %r', name)
+                return True
+            state = self.wait_for_state(name, lambda s: s == 'FAULT')
+            if not state:
+                self._log.warn('failed to kill %r', name)
+                return False
         self._log.info('killed %r', name)
         return True
 
@@ -656,8 +669,8 @@ class Starter:
             try:
                 self.starter_device.DevStop(name)
             except tango.DevFailed as err:
-                err = err[0].desc
-                self._log.warning('failed to execute stop on %r (%r)', name, err)
+                err = err.args[0].desc
+                self._log.warning('failed to stop on %r (%r)', name, err)
                 return False
             state = self.wait_for_state(name, lambda s: s in ('MOVING', 'FAULT'))
             if not state:
@@ -673,6 +686,19 @@ class Starter:
         self._log.info('stopped %r', name)
         return True
 
+    def stop_kill_server(self, name, timeout=5, dry_run=False):
+        task = gevent.spawn(self.stop_server, name, dry_run=dry_run)
+        stopped = False
+        try:
+            stopped = task.get(timeout=timeout)
+        except gevent.Timeout:
+            self._log.warning(
+                'timeout trying to stop %s. Performing a kill', name)
+            stopped = False
+            task.kill()
+        if not stopped:
+            self.kill_server(name, dry_run=dry_run)
+
     def start_server(self, name, dry_run=False):
         state = self[name]['state']
         if state == 'ON':
@@ -684,7 +710,7 @@ class Starter:
             self.starter_device.DevStart(name)
             state = self.wait_for_state(name, lambda s: s in ('MOVING', 'ON'))
             if not state:
-                self._log.warn('failed to wait to start %r', name)
+                self._log.warn('failed to start %r', name)
                 return False
             if state == 'ON':
                 self._log.info('started %r', name)
@@ -745,13 +771,13 @@ def find_starter_for_server(server_name, db=None):
 
 
 def find_starters_for_servers(names, db=None):
-    starters = starters(db=db).values()
+    starters_map = starters(db=db).values()
     result = collections.defaultdict(list)
-    for starter, servers in starters_servers(*starters).items():
+    for starter, servers in starters_servers(*starters_map).items():
         servers = servers or {}
         for name in names:
             if name in servers:
-                result[starter].append(server[name])
+                result[starter].append(servers[name])
     return result
 
 
@@ -814,7 +840,8 @@ def stop_server(name, db=None):
 
 def starter_stop_servers(
         starter_name=None, server_name=None, server_type=None,
-        server_instance=None, timeout=None, dry_run=True, db=None):
+        server_instance=None, timeout=None, stop_timeout=5,
+        dry_run=False, db=None):
 
     with gevent.Timeout(timeout):
         levels = find_starters_levels_for_filtered_servers(
@@ -825,8 +852,9 @@ def starter_stop_servers(
         for level in sorted(levels, reverse=True):
             level_tasks = []
             for starter, servers in levels[level].items():
-                stop = starter.stop_server
-                tasks = [gevent.spawn(stop, server['name'], dry_run=dry_run)
+                stop = starter.stop_kill_server
+                tasks = [gevent.spawn(stop, server['name'],
+                                      timeout=stop_timeout, dry_run=dry_run)
                          for server in servers]
                 level_tasks.extend(tasks)
             log.info('beginning to stop servers at level {}'.format(level))
@@ -836,7 +864,7 @@ def starter_stop_servers(
 
 def starter_start_servers(
         starter_name=None, server_name=None, server_type=None,
-        server_instance=None, timeout=None, dry_run=True, db=None):
+        server_instance=None, timeout=None, dry_run=False, db=None):
 
     with gevent.Timeout(timeout):
         levels = find_starters_levels_for_filtered_servers(
@@ -858,7 +886,7 @@ def starter_start_servers(
 
 def starter_restart_servers(
         starter_name=None, server_name=None, server_type=None,
-        server_instance=None, timeout=None, dry_run=True, db=None):
+        server_instance=None, timeout=None, dry_run=False, db=None):
 
     db = get_db(db)
 
